/* normalmode_chown.cc
 * This file belongs to Worker, a file manager for UN*X/X11.
 * Copyright (C) 2011-2013 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "normalmode.h"
#include "worker_locale.h"
#include "fileentry.hh"
#include "nmcopyopdir.hh"
#include "nmfieldlistviewdnd.hh"
#include "verzeichnis.hh"
#include "simplelist.hh"
#include <aguix/button.h>
#include "copycore.hh"
#include "copyopwin.hh"
#include "chownorder.hh"
#include "chmodowncore.hh"

void NormalMode::chownf(struct NM_chownorder *coorder)
{
    if ( coorder == NULL ) return;
    if ( getCurrentDir() == NULL ) return;

    finishsearchmode();
  
    disableDirWatch();
  
    ChModOwnCore changecore( NULL, coorder, aguix );

    {
        std::list< NM_specialsourceInt *> colist;
        switch ( coorder->source ) {
            case NM_chownorder::NM_SPECIAL:
                buildSpecialSourceIntList( *( coorder->sources ),
                                           colist );
                break;
            case NM_chownorder::NM_ONLYACTIVE:
                getSelFiles( &colist, LM_GETFILES_ONLYACTIVE, false );
                break;
            default:  // all selected entries
                getSelFiles( &colist, LM_GETFILES_SELORACT, false );
                break;
        }
        for ( auto &it : colist ) {
            changecore.registerFEToChange( *it->entry(), it->row );
        }

        NM_specialsourceInt *ss1;
        std::list<NM_specialsourceInt*>::iterator iti1;

        for ( iti1 = colist.begin(); iti1 != colist.end(); iti1++ ) {
            ss1 = *iti1;
            if(ss1->cod!=NULL) delete ss1->cod;
            delete ss1;
        }
    }

    changecore.setPostChangeCallback( [this]( const FileEntry &fe, int row,
                                              ChModOwnCore::nm_change_t res )
                                      {
                                          if ( res == ChModOwnCore::NM_CHANGE_OK ) {
                                              FileEntry *real_fe = getFEForRow( row );

                                              if ( real_fe && real_fe->equals( fe ) ) {
                                                  deselect( real_fe, row );
                                              }
                                          }
                                      } );

    changecore.executeChOwn();

    update(false);

    enableDirWatch();
}
